asm certifierRef1

import ../../STDL/StandardLibrary
import ../../STDL/CTLlibrary

signature:
	enum domain Answers={SKIP| EXIT | WRONG | RIGHT}  //user answers: SKIP, EXIT, RIGHT, WRONG
	enum domain OutMessage={CERTIFICATE | NOTCERTIFICATE} // Certificate --> level certified, NotCertificate --> level not certified
	domain Level subsetof Integer //level the user is certifying
	domain Certificate subsetof Integer // level certified

	dynamic controlled test: Boolean //true:test is running false:test is finished
	dynamic controlled outMessage: OutMessage //out message: Certificate --> level certified, NotCertificate --> level not certified
	dynamic monitored getAnswer: Answers //user answer --> it is monitored
	dynamic controlled levelTest: Level //level the user is certifying
	dynamic controlled levelCertificate: Certificate // level certified
	dynamic controlled loop: Boolean /* Memorized if the user gone up of at least one level during certification process*/
	dynamic controlled answerSkip: Boolean // it is allowed only one skip for each level
										// if the user skips twice the test goes up of one level

definitions:
	domain Level={1..6}
	domain Certificate={1..7} //7 not certified 1..6 level certified

	macro rule r_goOut=
		par
			test:=false
			levelCertificate:=7
		endpar

	macro rule r_goUp=
		par
			levelTest:=levelTest+1
			answerSkip:=false	
			levelCertificate:=levelCertificate+1
		endpar

	
	macro rule r_goUpCheckLevel=
		if levelTest<6 then
			r_goUp[]
		else
			test:=false
		endif
	
	macro rule r_wrongAnswer=
		par
			loop:=true
			r_goUpCheckLevel[]
		endpar
		
	macro rule r_rightAnswNoLoop=
		par
			answerSkip:=false
			levelTest:=levelTest-1
			levelCertificate:=levelCertificate-1
		endpar

	macro rule r_certificate=
		par
			levelCertificate:=levelCertificate-1
			test:=false
		endpar

	macro rule r_goDown=
		if loop=false then
			r_rightAnswNoLoop[]
		else
			r_certificate[]
		endif

	macro rule r_rightAnswer=
		if levelTest>1 then
			r_goDown[]
		else 
			r_certificate[]
		endif

	macro rule r_checkanswer($a in Answers)=
		if ($a = RIGHT) then
			r_rightAnswer[]
		else
			r_wrongAnswer[]
		endif

	macro rule r_skip=
		if (answerSkip) then
			r_wrongAnswer[]
		else
			answerSkip:=true
		endif

	macro rule r_test=	
		if (getAnswer=EXIT) then
			r_goOut[]
		else
			if (getAnswer=SKIP) then
				r_skip[]
			else
			r_checkanswer[getAnswer]
			endif
		endif

	macro rule r_exit=
		if levelCertificate=7 then
			outMessage:=NOTCERTIFICATE
		else
	   		outMessage:=CERTIFICATE
		endif

	CTLSPEC af(not test)
	CTLSPEC af(outMessage=CERTIFICATE) or ef (outMessage=NOTCERTIFICATE)
	CTLSPEC not(ef (not test and levelTest=1)) 
	CTLSPEC ag(test)
	CTLSPEC not ef (not test and outMessage=CERTIFICATE)
	
	invariant over levelTest: levelCertificate>=levelTest

	main rule r_Main =
		if (test) then
			r_test[]	
		else
			r_exit[]
		endif
		
default init s0:
	function test = true
	function levelTest=6
	function levelCertificate=7
	function answerSkip=false
	function loop=false
