asm certifierRef3

import ../../STDL/StandardLibrary
import ../../STDL/CTLlibrary

signature:
	enum domain Answers={SKIP | EXIT | RIGHT | WRONG}
	enum domain OutMessage={CERTIFICATE | NOTCERTIFICATE}
	domain Level subsetof Integer
	domain Certificate subsetof Integer
	domain RightAnswer subsetof Integer

	dynamic controlled test: Boolean
	dynamic controlled outMessage: OutMessage
	dynamic monitored getAnswer: Answers
	dynamic controlled levelTest: Level
	dynamic controlled levelCertificate: Certificate
	dynamic controlled rightAnswer: RightAnswer
	dynamic controlled answerError: Boolean
	dynamic controlled answerSkip: Boolean
	dynamic controlled loop: Boolean

definitions:
	domain Level={1..6}
	domain Certificate={1..7}
	domain RightAnswer={0..2}

	macro rule r_goOut =
		par
			test:=false
			levelCertificate:=7
		endpar

	macro rule r_goUp =
		par
			levelTest:=levelTest+1
			answerError:=false
			answerSkip:=false	
			rightAnswer:=0
			levelCertificate:=levelCertificate+1
		endpar

	macro rule r_goUpCheckLevel =
		if levelTest<6 then
			r_goUp[]
		else
			test:=false
		endif

	macro rule r_wrong =
		par
			loop:=true
			r_goUpCheckLevel[]
		endpar

	macro rule r_wrongAnswer =
		if (answerError) then
			r_wrong[]
		else
			answerError:=true
		endif

	macro rule r_rightAnswNoLoop =
		par
			answerError:=false
			answerSkip:=false
			levelTest:=levelTest-1
			levelCertificate:=levelCertificate-1
		endpar

	macro rule r_certificate =
		if rightAnswer<=1 then
			rightAnswer:=rightAnswer+1
		else
			par
				levelCertificate:=levelCertificate-1
				test:=false
			endpar
		endif
		
	macro rule r_goDown =
		if loop=false then
			r_rightAnswNoLoop[]
		else
			r_certificate[]
		endif

	macro rule r_rightAnswer =
		if levelTest>1 then
			r_goDown[]
		else 
			r_certificate[]
		endif

	macro rule r_checkanswer($a in Answers) =
		if ($a = RIGHT) then
			r_rightAnswer[]
		else
			r_wrongAnswer[]
		endif

	macro rule r_skip=
		par
			if (answerSkip) then
				r_wrong[]
			else
				answerSkip:=true
			endif
			skip
		endpar

	macro rule r_test =
		if (getAnswer=EXIT) then
			r_goOut[]
		else
			if (getAnswer=SKIP) then
				r_skip[]
			else
				r_checkanswer[getAnswer]
			endif
		endif

	macro rule r_exit =
		if levelCertificate=7 then
			outMessage:=NOTCERTIFICATE
		else
	   		outMessage:=CERTIFICATE
		endif

	CTLSPEC af(not test)
	CTLSPEC af(outMessage=CERTIFICATE) or ef (outMessage=NOTCERTIFICATE)
	CTLSPEC not(ef (not test and levelTest=1)) 
	CTLSPEC ag(test)
	CTLSPEC not ef (not test and outMessage=CERTIFICATE)

	invariant over levelTest: levelCertificate>=levelTest

	main rule r_Main =
		if (test) then
			r_test[]	
		else
			r_exit[]
		endif
		
default init s0:
	function test = true
	function levelTest=6
	function levelCertificate=7
	function rightAnswer=0
	function answerError=false
	function answerSkip=false
	function loop=false
